// ne2000cr.h

#define neRESETwr   0x1f
#define neDMArw     0x10

//
// NE2000 writable registers:
//
// page 0
//
#define neCRwr     0x00
#define nePSTARTwr 0x01
#define nePSTOPwr  0x02
#define neBNRYwr   0x03
#define neTPSRwr   0x04
#define neTBCR0wr  0x05
#define neTBCR1wr  0x06
#define neISRwr    0x07
#define neRSAR0wr  0x08
#define neRSAR1wr  0x09
#define neRBCR0wr  0x0a
#define neRBCR1wr  0x0b
#define neRCRwr    0x0c
#define neTCRwr    0x0d
#define neDCRwr    0x0e
#define neIMRwr    0x0f
//
// page 1
//
#define neCRwr     0x00
#define nePAR0wr   0x01
#define nePAR1wr   0x02
#define nePAR2wr   0x03
#define nePAR3wr   0x04
#define nePAR4wr   0x05
#define nePAR5wr   0x06
#define neCURRwr   0x07
#define neMAR0wr   0x08
#define neMAR1wr   0x09
#define neMAR2wr   0x0a
#define neMAR3wr   0x0b
#define neMAR4wr   0x0c
#define neMAR5wr   0x0d
#define neMAR6wr   0x0e
#define neMAR7wr   0x0f
//
// page 2
//
#define neCRwr     0x00
#define neCLDA0wr  0x01
#define neCLDA1wr  0x02
//
// page 3
//
#define neCRwr     0x00
#define neCRDwr    0x07

//
// NE2000 readable registers
//
// page 0
//
#define neCRrd     0x00
#define neCLDA0rd  0x01
#define neCLDA1rd  0x02
#define neBNRYrd   0x03
#define neTSRrd    0x04
#define neNCRrd    0x05
#define neFIFOrd   0x06
#define neISRrd    0x07
#define neCRDA0rd  0x08
#define neCRDA1rd  0x09
#define neCRArd    0x0a
#define neCRBrd    0x0b
#define neRSRrd    0x0c
#define neCNTR0rd  0x0d
#define neCNTR1rd  0x0e
#define neCNTR2rd  0x0f
//
// page 1
//
#define neCRrd     0x00
#define nePAR0rd   0x01
#define nePAR1rd   0x02
#define nePAR2rd   0x03
#define nePAR3rd   0x04
#define nePAR4rd   0x05
#define nePAR5rd   0x06
#define neCURRrd   0x07
#define neMAR0rd   0x08
#define neMAR1rd   0x09
#define neMAR2rd   0x0a
#define neMAR3rd   0x0b
#define neMAR4rd   0x0c
#define neMAR5rd   0x0d
#define neMAR6rd   0x0e
#define neMAR7rd   0x0f
//
// page 2
//
#define neCRrd     0x00
#define nePSTARTrd 0x01
#define nePSTOPrd  0x02
#define neCRCrd    0x0b
#define neRCRrd    0x0c
#define neTCRrd    0x0d
#define neDCRrd    0x0e
#define neIMRrd    0x0f
//
// page 3
//
#define neCRrd     0x00
#define neCRDrd    0x07

//
// NE2000 register bit assignments
//
#define neCR_STP (1<<0)
#define neCR_STA (1<<1)
#define neCR_TXP (1<<2)
#define neCR_RD0 (1<<3)
#define neCR_RD1 (1<<4)
#define neCR_RD2 (1<<5)
#define neCR_PS0 (1<<6)
#define neCR_PS1 (1<<7)

#define neDCR_WTS (1<<0)
#define neDCR_BOS (1<<1)
#define neDCR_LAS (1<<2)
#define neDCR_LS  (1<<3)
#define neDCR_ARM (1<<4)
#define neDCR_FT0 (1<<5)
#define neDCR_FT1 (1<<6)

#define neTCR_CRC (1<<0)
#define neTCR_LB0 (1<<1)
#define neTCR_LB1 (1<<2)
#define neTCR_ATD (1<<3)
#define neTCR_OFST (1<<4)

#define neTSR_PTX (1<<0)
#define neTSR_COL (1<<2)
#define neTSR_ABT (1<<3)
#define neTSR_CRS (1<<4)
#define neTSR_FU  (1<<5)
#define neTSR_CDH (1<<6)
#define neTSR_OWC (1<<7)

#define neRCR_SEP (1<<0)
#define neRCR_ARP (1<<1)
#define neRCR_ABP (1<<2)
#define neRCR_AM  (1<<3)
#define neRCR_PRO (1<<4)
#define neRCR_MON (1<<5)

#define neRSR_PRX (1<<0)
#define neRSR_CRC (1<<1)
#define neRSR_FAE (1<<2)
#define neRSR_FO  (1<<3)
#define neRSR_MPA (1<<4)
#define neRSR_PHY (1<<5)
#define neRSR_DIS (1<<6)
#define neRSR_DFR (1<<7)

#define neIMR_PRXE (1<<0)
#define neIMR_PTXE (1<<1)
#define neIMR_RXEE (1<<2)
#define neIMR_TXEE (1<<3)
#define neIMR_OVWE (1<<4)
#define neIMR_CNTE (1<<5)
#define neIMR_RDCE (1<<6)

#define neISR_PRX (1<<0)
#define neISR_PTX (1<<1)
#define neISR_RXE (1<<2)
#define neISR_TXE (1<<3)
#define neISR_OVW (1<<4)
#define neISR_CNT (1<<5)
#define neISR_RDC (1<<6)
#define neISR_RST (1<<7)

//
// NE2000 simulated registers
//
extern unsigned char neCR;
extern unsigned char neCLDA0;
extern unsigned char neCLDA1;
extern unsigned char nePSTART;
extern unsigned char nePSTOP;
extern unsigned char neBNRY;
extern unsigned char neTSR;
extern unsigned char neTPSR;
extern unsigned char neNCR;
extern unsigned char neFIFO;
extern unsigned char neTBCR0;
extern unsigned char neTBCR1;
extern unsigned char neISR;
extern unsigned char neCRDA0;
extern unsigned char neCRDA1;
extern unsigned char neRSAR0;
extern unsigned char neRSAR1;
extern unsigned char neRBCR0;
extern unsigned char neRBCR1;
extern unsigned char neRSR;
extern unsigned char neRCR;
extern unsigned char neCNTR0;
extern unsigned char neCNTR1;
extern unsigned char neCNTR2;
extern unsigned char neTCR;
extern unsigned char neDCR;
extern unsigned char neIMR;
extern unsigned char nePAR0;
extern unsigned char nePAR1;
extern unsigned char nePAR2;
extern unsigned char nePAR3;
extern unsigned char nePAR4;
extern unsigned char nePAR5;
extern unsigned char neCURR;
extern unsigned char neMAR0;
extern unsigned char neMAR1;
extern unsigned char neMAR2;
extern unsigned char neMAR3;
extern unsigned char neMAR4;
extern unsigned char neMAR5;
extern unsigned char neMAR6;
extern unsigned char neMAR7;
extern unsigned char neCRA;
extern unsigned char neCRB;
extern unsigned char neCRC;
extern unsigned char neCRD;

#define NE_RAM_PAGES 32        /* number of 256-bytes packet buffer pages */
#define NE_RAM_OFFSET 0x4000   /* offset to start of packet buffers */

#define NE_RAM_BYTES (NE_RAM_PAGES * 256)
#define NE_ADDR_MAX (NE_RAM_BYTES + NE_RAM_OFFSET)
extern unsigned char nePacketBuf[NE_ADDR_MAX];

#define NE_CHK_ADDR(addr) \
    (((addr) < 0 || (addr) >= NE_ADDR_MAX) ? 0 : 1)

#define NE_RAM_GET(addr) \
    (NE_CHK_ADDR(addr) ? nePacketBuf[addr] : 0xff)

#define NE_RAM_PUT(addr, val) \
    if (NE_CHK_ADDR(addr)) nePacketBuf[addr] = (val)

